'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.requireModule = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _path = require('path');

var _module = require('./module');

var _module2 = _interopRequireDefault(_module);

var _plugins = require('./plugins');

var _mocks = require('./mocks');

var _globals = require('./globals');

var _globals2 = _interopRequireDefault(_globals);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var thisModule = module;

var patternMatch = function patternMatch(fileName) {
  return function (pattern) {
    if (typeof pattern == 'function') {
      return pattern(fileName);
    }
    return fileName.match(pattern);
  };
};

var requireModule = exports.requireModule = function requireModule(name) {
  if (typeof __webpack_require__ !== 'undefined') {
    return __webpack_require__(name);
  } else {
    return require(name);
  }
};

var testPassby = function testPassby(request, module) {
  var _getScope = (0, _globals2.default)(),
      parentModule = _getScope.parentModule,
      passBy = _getScope.passBy,
      mockedModules = _getScope.mockedModules,
      isolation = _getScope.isolation;

  // was called from test


  if ((0, _module.moduleCompare)(module, parentModule)) {
    //if (module === parentModule || module == module.parent) {
    return true;
  }
  // if parent is in the pass list - pass everything
  var fileName = _module2.default._resolveFilename(request, module);
  var m = module;

  var test = function test(fileName) {
    return !isolation.noAutoPassBy && mockedModules[fileName] || // parent was mocked
    passBy.filter(patternMatch(fileName)).length // parent is in pass list
    ;
  };

  while (m) {
    if (test(fileName)) {
      return true;
    }
    fileName = (0, _module.getModuleName)(m);
    m = (0, _module.getModuleParent)(m);
  }
  return test(fileName);
};

function mockResult(name, data) {
  if (!data.default) {
    data.default = data;
  }
  return data;
}

function monkeyPatchPath(addr) {
  var path = addr.split('/');
  if (path[0] == '..') {
    path[0] = '.';
    return path.join('/');
  }
  return addr;
}

function mockLoader(request, parent, isMain) {
  var _getScope2 = (0, _globals2.default)(),
      parentModule = _getScope2.parentModule,
      mockedModules = _getScope2.mockedModules,
      isolation = _getScope2.isolation;

  var baseRequest = _module2.default._resolveFilename(request, parent);
  var shortRequest = monkeyPatchPath((0, _path.relative)((0, _module.getModuleName)(parent), request));

  if ((0, _module.moduleCompare)(parent, parentModule) || (0, _module.moduleCompare)(parent, thisModule)) {
    delete _module2.default._cache[baseRequest];
    mockedModules[baseRequest] = true;
  }

  var mock = (0, _mocks.getMock)(baseRequest) || (0, _mocks.getMock)(request) || (0, _mocks.getMock)(shortRequest);

  if (mock) {
    if ((0, _plugins.shouldMock)(mock, request, parent, parentModule)) {
      // this file fill be not cached, but it`s opener - will. And we have to remember it
      mockedModules[(0, _module.getModuleName)(parent)] = true;
      mock.usedAs = mock.usedAs || [];
      mock.usedAs.push(baseRequest);

      mockedModules[baseRequest] = true;

      if (mock.allowCallThrough) {
        if (!mock.original) {
          mock.original = (0, _module.originalLoader)(request, parent, isMain);
        }
      }

      if (mock.overrideBy) {
        if (!mock.override) {
          if (typeof mock.overrideBy === 'string') {
            mock.override = (0, _module.originalLoader)((0, _module.pickModuleName)(mock.overrideBy, parent), parent, isMain);
          } else {
            mock.override = mock.overrideBy({
              name: request,
              fullName: baseRequest,
              parent: parent,
              original: mock.original,
              requireActual: function requireActual(name) {
                return (0, _module.originalLoader)((0, _module.pickModuleName)(name, parent), parent, isMain);
              }
            });
          }
        }
        return mockResult(request, mock.override);
      }

      if (mock.allowCallThrough) {
        if (typeof mock.original === 'function') {
          if (_typeof(mock.value) === 'object' && Object.keys(mock.value).length === 0) {
            return mockResult(request, mock.original);
          } else {
            throw new Error('rewiremock: trying to merge Functional base with callThrough mock at ' + request + '. Use overrideBy instead.');
          }
        }
        return mockResult(request, Object.assign({}, mock.original, mock.value, { __esModule: mock.original.__esModule }));
      }
      return mockResult(request, mock.value);
    } else {
      // why you shouldn't?
    }
  }

  if (isolation && !mockedModules[baseRequest]) {
    if (!testPassby(request, parent)) {
      throw new Error('rewiremock: isolation breach by [' + request + ']. Requested from ', (0, _module.getModuleName)(parent));
    }
  }

  return (0, _module.originalLoader)(request, parent, isMain);
}

exports.default = mockLoader;